@@ -28,6 +28,8 @@
 osi_rwlock_t cm_connLock;
 
 long RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
+long ConnDeadtimeout = CM_CONN_CONNDEADTIME;
+long HardDeadtimeout = CM_CONN_HARDDEADTIME;
 
 #define LANMAN_WKS_PARAM_KEY "SYSTEM\\CurrentControlSet\\Services\\lanmanworkstation\\parameters"
 #define LANMAN_WKS_SESSION_TIMEOUT "SessTimeout"
@@ -52,8 +54,11 @@ void cm_InitConn(void)
 		lock_InitializeRWLock(&cm_connLock, "connection global lock");
 
         /* keisa - read timeout value for lanmanworkstation  service.
-         * It is used as hardtimeout for connections. 
-         * Default value is 45 
+         * jaltman - as per 
+         *   http://support.microsoft.com:80/support/kb/articles/Q102/0/67.asp&NoWebContent=1
+         * the SessTimeout is a minimum timeout not a maximum timeout.  Therefore, 
+         * I believe that the default should not be short.  Instead, we should wait until
+         * RX times out before reporting a timeout to the SMB client.
          */
 		code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, LANMAN_WKS_PARAM_KEY,
                             0, KEY_QUERY_VALUE, &parmKey);
@@ -66,13 +71,17 @@ void cm_InitConn(void)
             {
                 afsi_log("lanmanworkstation : SessTimeout %d", sessTimeout);
                 RDRtimeout = sessTimeout;
-            }
-		    else
-            {
-                RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
+                if ( ConnDeadtimeout < RDRtimeout + 15 ) {
+                    ConnDeadtimeout = RDRtimeout + 15;
+                    afsi_log("ConnDeadTimeout increased to %d", ConnDeadtimeout);
+                }
+                if ( HardDeadtimeout < 2 * ConnDeadtimeout ) {
+                    HardDeadtimeout = 2 * ConnDeadtimeout;
+                    afsi_log("HardDeadTimeout increased to %d", HardDeadtimeout);
+                }
             }
         }
-		
+
         osi_EndOnce(&once);
     }
 }
@@ -85,39 +94,29 @@ void cm_InitReq(cm_req_t *reqp)
 #else
         gettimeofday(&reqp->startTime, NULL);
 #endif
- 
 }
 
-long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
-	struct cm_req *reqp, cm_serverRef_t **serverspp)
+static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
+	struct cm_req *reqp, cm_serverRef_t ***serversppp)
 {
 	long code;
-        cm_volume_t *volp = NULL;
-        cm_serverRef_t *serversp = NULL;
-        cm_cell_t *cellp = NULL;
+    cm_volume_t *volp = NULL;
+    cm_cell_t *cellp = NULL;
 
-        if (!fidp) {
-		*serverspp = NULL;
+    if (!fidp) {
+		*serversppp = NULL;
 		return 0;
 	}
 
 	cellp = cm_FindCellByID(fidp->cell);
-        if (!cellp) return CM_ERROR_NOSUCHCELL;
+    if (!cellp) return CM_ERROR_NOSUCHCELL;
 
-        code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, &volp);
-        if (code) return code;
-        
-	if (fidp->volume == volp->rwID)
-        	serversp = volp->rwServersp;
-	else if (fidp->volume == volp->roID)
-        	serversp = volp->roServersp;
-	else if (fidp->volume == volp->bkID)
-        	serversp = volp->bkServersp;
-	else
-		serversp = NULL;
+    code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, &volp);
+    if (code) return code;
+    
+    *serversppp = cm_GetVolServers(volp, fidp->volume);
 
-        cm_PutVolume(volp);
-	*serverspp = serversp;
+    cm_PutVolume(volp);
 	return 0;
 }
 
@@ -139,14 +138,19 @@ long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
  */
 int
 cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
-	struct cm_fid *fidp,
-	AFSVolSync *volSyncp, cm_callbackRequest_t *cbrp, long errorCode)
+	struct cm_fid *fidp, 
+	AFSVolSync *volSyncp, 
+	cm_serverRef_t * serversp,
+	cm_callbackRequest_t *cbrp, long errorCode)
 {
-	cm_server_t *serverp;
-	cm_serverRef_t *serversp, *tsrp;
+    cm_server_t *serverp = 0;
+    cm_serverRef_t **serverspp = 0;
+	cm_serverRef_t *tsrp;
 	cm_ucell_t *ucellp;
-        int retry = 0;
+    int retry = 0;
+    int free_svr_list = 0;
 	int dead_session;
+    long timeUsed, timeLeft;
         
 	osi_Log2(afsd_logp, "cm_Analyze connp 0x%x, code %d",
 		 (long) connp, errorCode);
@@ -158,17 +162,29 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 		serverp = connp->serverp;
 
 	/* Update callback pointer */
-        if (cbrp && errorCode == 0) cbrp->serverp = connp->serverp;
+    if (cbrp && serverp && errorCode == 0) {
+        if (cbrp->serverp) {
+            if ( cbrp->serverp != serverp ) {
+                lock_ObtainWrite(&cm_serverLock);
+                cm_PutServerNoLock(cbrp->serverp);
+                cm_GetServerNoLock(serverp);
+                lock_ReleaseWrite(&cm_serverLock);
+            }
+        } else {
+            cm_GetServer(serverp);
+        }
+        lock_ObtainWrite(&cm_callbackLock);
+        cbrp->serverp = serverp;
+        lock_ReleaseWrite(&cm_callbackLock);
+    }
 
 	/* If not allowed to retry, don't */
 	if (reqp->flags & CM_REQ_NORETRY)
 		goto out;
 
-	/* if timeout - check that is did not exceed the SMB timeout
-	   and retry */
-	if (errorCode == CM_ERROR_TIMEDOUT)
-    {
-	    long timeUsed, timeLeft;
+	/* if timeout - check that it did not exceed the SMB timeout
+     * and retry */
+    
 	    /* timeleft - get if from reqp the same way as cmXonnByMServers does */
 #ifndef DJGPP
 	    timeUsed = (GetCurrentTime() - reqp->startTime) / 1000;
@@ -179,37 +195,75 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 	    
 	    /* leave 5 seconds margin for sleep */
 	    timeLeft = RDRtimeout - timeUsed;
-	    if (timeLeft > 5)
-        {
+
+    if (errorCode == CM_ERROR_TIMEDOUT && timeLeft > 5 ) {
             thrd_Sleep(3000);
             cm_CheckServers(CM_FLAG_CHECKDOWNSERVERS, NULL);
             retry = 1;
         } 
-    }
 
     /* if all servers are offline, mark them non-busy and start over */
-	if (errorCode == CM_ERROR_ALLOFFLINE) {
+    if (errorCode == CM_ERROR_ALLOFFLINE && timeLeft > 7) {
 	    osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLOFFLINE.");
 	    thrd_Sleep(5000);
 	    /* cm_ForceUpdateVolume marks all servers as non_busy */
-	    cm_ForceUpdateVolume(fidp, userp, reqp);
-	    retry = 1;
+		/* No it doesn't and it won't do anything if all of the 
+		 * the servers are marked as DOWN.  So clear the DOWN
+		 * flag and reset the busy state as well.
+		 */
+        if (!serversp) {
+            cm_GetServerList(fidp, userp, reqp, &serverspp);
+            serversp = *serverspp;
+            free_svr_list = 1;
+        }
+        if (serversp) {
+            lock_ObtainWrite(&cm_serverLock);
+            for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+                tsrp->server->flags &= ~CM_SERVERFLAG_DOWN;
+                if (tsrp->status == busy)
+                    tsrp->status = not_busy;
+            }
+            lock_ReleaseWrite(&cm_serverLock);
+            if (free_svr_list) {
+                cm_FreeServerList(&serversp);
+                *serverspp = serversp;
+            }
+            retry = 1;
+        }
+
+        if (fidp != NULL)   /* Not a VLDB call */
+            cm_ForceUpdateVolume(fidp, userp, reqp);
 	}
 
 	/* if all servers are busy, mark them non-busy and start over */
-	if (errorCode == CM_ERROR_ALLBUSY) {
-		cm_GetServerList(fidp, userp, reqp, &serversp);
+    if (errorCode == CM_ERROR_ALLBUSY && timeLeft > 7) {
+        thrd_Sleep(5000);
+        if (!serversp) {
+            cm_GetServerList(fidp, userp, reqp, &serverspp);
+            serversp = *serverspp;
+            free_svr_list = 1;
+        }
+		lock_ObtainWrite(&cm_serverLock);
 		for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
 			if (tsrp->status == busy)
 				tsrp->status = not_busy;
 		}
-		thrd_Sleep(5000);
+        lock_ReleaseWrite(&cm_serverLock);
+        if (free_svr_list) {
+            cm_FreeServerList(&serversp);
+            *serverspp = serversp;
+        }
 		retry = 1;
 	}
 
 	/* special codes:  VBUSY and VRESTARTING */
 	if (errorCode == VBUSY || errorCode == VRESTARTING) {
-		cm_GetServerList(fidp, userp, reqp, &serversp);
+        if (!serversp) {
+            cm_GetServerList(fidp, userp, reqp, &serverspp);
+            serversp = *serverspp;
+            free_svr_list = 1;
+        }
+		lock_ObtainWrite(&cm_serverLock);
 		for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
 			if (tsrp->server == serverp
 			    && tsrp->status == not_busy) {
@@ -217,6 +271,11 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 				break;
 			}
 		}
+        lock_ReleaseWrite(&cm_serverLock);
+        if (free_svr_list) {
+            cm_FreeServerList(&serversp);
+            *serverspp = serversp;
+        }
 		retry = 1;
 	}
 
@@ -248,12 +307,20 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 		}
 
 		/* Mark server offline for this volume */
-		cm_GetServerList(fidp, userp, reqp, &serversp);
-
+        if (!serversp) {
+            cm_GetServerList(fidp, userp, reqp, &serverspp);
+            serversp = *serverspp;
+            free_svr_list = 1;
+        }
 		for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
 			if (tsrp->server == serverp)
 				tsrp->status = offline;
 		}
+        if (free_svr_list) {
+            cm_FreeServerList(&serversp);
+            *serverspp = serversp;
+        }
+        if ( timeLeft > 2 )
 		retry = 1;
 	}
 
@@ -282,10 +349,11 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 	else if (errorCode >= -64 && errorCode < 0) {
 		/* mark server as down */
 		lock_ObtainMutex(&serverp->mx);
-                serverp->flags |= CM_SERVERFLAG_DOWN;
+        serverp->flags |= CM_SERVERFLAG_DOWN;
 		lock_ReleaseMutex(&serverp->mx);
-                retry = 1;
-        }
+            if ( timeLeft > 2 )
+        retry = 1;
+    }
 
 	if (errorCode == RXKADEXPIRED && !dead_session) {
 		lock_ObtainMutex(&userp->mx);
@@ -297,6 +365,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 		ucellp->flags &= ~CM_UCELLFLAG_RXKAD;
 		ucellp->gen++;
 		lock_ReleaseMutex(&userp->mx);
+            if ( timeLeft > 2 )
 		retry = 1;
 	}
 
@@ -315,39 +384,33 @@ out:
 long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
 	cm_req_t *reqp, cm_conn_t **connpp)
 {
-	long code;
-	cm_serverRef_t *tsrp;
-        cm_server_t *tsp;
-        long firstError = 0;
-	int someBusy = 0, someOffline = 0, allDown = 1;
-	long timeUsed, timeLeft, hardTimeLeft;
+    long code;
+    cm_serverRef_t *tsrp;
+    cm_server_t *tsp;
+    long firstError = 0;
+    int someBusy = 0, someOffline = 0, allBusy = 1, allDown = 1;
+    long timeUsed, timeLeft, hardTimeLeft;
 #ifdef DJGPP
-        struct timeval now;
+    struct timeval now;
 #endif /* DJGPP */        
 
-        *connpp = NULL;
+    *connpp = NULL;
 
 #ifndef DJGPP
-	timeUsed = (GetCurrentTime() - reqp->startTime) / 1000;
+    timeUsed = (GetCurrentTime() - reqp->startTime) / 1000;
 #else
-        gettimeofday(&now, NULL);
-        timeUsed = sub_time(now, reqp->startTime) / 1000;
+    gettimeofday(&now, NULL);
+    timeUsed = sub_time(now, reqp->startTime) / 1000;
 #endif
         
-	/* leave 5 seconds margin of safety */
-	timeLeft = RDRtimeout - timeUsed - 5;
-	hardTimeLeft = timeLeft;
+    /* leave 5 seconds margin of safety */
+    timeLeft =  ConnDeadtimeout - timeUsed - 5;
+    hardTimeLeft = HardDeadtimeout - timeUsed - 5;
 
-	/* Time enough to do an RPC? */
-	if (timeLeft < 1) {
-		return CM_ERROR_TIMEDOUT;
-	}
-
-	lock_ObtainWrite(&cm_serverLock);
-
-    for(tsrp = serversp; tsrp; tsrp=tsrp->next) {
+    lock_ObtainWrite(&cm_serverLock);
+    for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
         tsp = tsrp->server;
-        tsp->refCount++;
+        cm_GetServerNoLock(tsp);
         lock_ReleaseWrite(&cm_serverLock);
         if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
             allDown = 0;
@@ -356,75 +419,73 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
             else if (tsrp->status == offline)
                 someOffline = 1;
             else {
+                allBusy = 0;
                 code = cm_ConnByServer(tsp, usersp, connpp);
                 if (code == 0) {
                     cm_PutServer(tsp);
                     /* Set RPC timeout */
-                    if (timeLeft > CM_CONN_CONNDEADTIME)
-                        timeLeft = CM_CONN_CONNDEADTIME;
+                    if (timeLeft > ConnDeadtimeout)
+                        timeLeft = ConnDeadtimeout;
 
-                    if (hardTimeLeft > CM_CONN_HARDDEADTIME) 
-                        hardTimeLeft = CM_CONN_HARDDEADTIME;
+                    if (hardTimeLeft > HardDeadtimeout) 
+                        hardTimeLeft = HardDeadtimeout;
 
                     lock_ObtainMutex(&(*connpp)->mx);
-                    rx_SetConnDeadTime((*connpp)->callp,
-                                        timeLeft);
-                    rx_SetConnHardDeadTime((*connpp)->callp, 
-                                            (u_short) hardTimeLeft);
+                    rx_SetConnDeadTime((*connpp)->callp, timeLeft);
+                    rx_SetConnHardDeadTime((*connpp)->callp, (u_short) hardTimeLeft);
                     lock_ReleaseMutex(&(*connpp)->mx);
-
                     return 0;
                 }
                 if (firstError == 0) 
                     firstError = code;
             }
-		} 
+        } 
         lock_ObtainWrite(&cm_serverLock);
-        osi_assert(tsp->refCount-- > 0);
+        cm_PutServerNoLock(tsp);
     }   
 
-	lock_ReleaseWrite(&cm_serverLock);
-	if (firstError == 0) {
-		if (someBusy) 
-			firstError = CM_ERROR_ALLBUSY;
-		else if (someOffline) 
-			firstError = CM_ERROR_ALLOFFLINE;
-		else if (!allDown && serversp) 
-			firstError = CM_ERROR_TIMEDOUT;
-		/* Only return CM_ERROR_NOSUCHVOLUME if there are no
-		   servers for this volume */
-		else 
-			firstError = CM_ERROR_NOSUCHVOLUME;
-	}
-	osi_Log1(afsd_logp, "cm_ConnByMServers returning %x", firstError);
+    lock_ReleaseWrite(&cm_serverLock);
+    if (firstError == 0) {
+        if (serversp == NULL)
+            firstError = CM_ERROR_NOSUCHVOLUME;
+        else if (allDown) 
+            firstError = CM_ERROR_ALLOFFLINE;
+        else if (allBusy) 
+            firstError = CM_ERROR_ALLBUSY;
+        else
+            firstError = CM_ERROR_TIMEDOUT;
+    }
+
+    osi_Log1(afsd_logp, "cm_ConnByMServers returning %x", firstError);
     return firstError;
 }
 
 /* called with a held server to GC all bad connections hanging off of the server */
 void cm_GCConnections(cm_server_t *serverp)
 {
-	cm_conn_t *tcp;
+    cm_conn_t *tcp;
     cm_conn_t **lcpp;
     cm_user_t *userp;
 
-	lock_ObtainWrite(&cm_connLock);
-	lcpp = &serverp->connsp;
-	for(tcp = *lcpp; tcp; tcp = *lcpp) {
-		userp = tcp->userp;
-		if (userp && tcp->refCount == 0 && (userp->vcRefs == 0)) {
-			/* do the deletion of this guy */
+    lock_ObtainWrite(&cm_connLock);
+    lcpp = &serverp->connsp;
+    for (tcp = *lcpp; tcp; tcp = *lcpp) {
+        userp = tcp->userp;
+        if (userp && tcp->refCount == 0 && (userp->vcRefs == 0)) {
+            /* do the deletion of this guy */
+            cm_PutServer(tcp->serverp);
             cm_ReleaseUser(userp);
             *lcpp = tcp->nextp;
-			rx_DestroyConnection(tcp->callp);
+            rx_DestroyConnection(tcp->callp);
             lock_FinalizeMutex(&tcp->mx);
             free(tcp);
         }
         else {
-			/* just advance to the next */
+            /* just advance to the next */
             lcpp = &tcp->nextp;
         }
     }
-	lock_ReleaseWrite(&cm_connLock);
+    lock_ReleaseWrite(&cm_connLock);
 }
 
 static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
@@ -434,25 +495,24 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
     int serviceID;
     int secIndex;
     struct rx_securityClass *secObjp;
-	afs_int32 level;
+    afs_int32 level;
 
-	if (serverp->type == CM_SERVER_VLDB) {
-		port = htons(7003);
+    if (serverp->type == CM_SERVER_VLDB) {
+        port = htons(7003);
         serviceID = 52;
     }
     else {
-		osi_assert(serverp->type == CM_SERVER_FILE);
+        osi_assert(serverp->type == CM_SERVER_FILE);
         port = htons(7000);
         serviceID = 1;
     }
-	if (ucellp->flags & CM_UCELLFLAG_RXKAD) {
-		secIndex = 2;
-		if (cryptall) {
-			level = rxkad_crypt;
-			tcp->cryptlevel = rxkad_crypt;
-		} else {
-			level = rxkad_clear;
-		}
+    if (ucellp->flags & CM_UCELLFLAG_RXKAD) {
+        secIndex = 2;
+        if (cryptall) {
+            level = tcp->cryptlevel = rxkad_crypt;
+        } else {
+            level = tcp->cryptlevel = rxkad_clear;
+        }
         secObjp = rxkad_NewClientSecurityObject(level,
                                                 &ucellp->sessionKey, ucellp->kvno,
                                                 ucellp->ticketLen, ucellp->ticketp);    
@@ -462,58 +522,65 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
         secIndex = 0;
         secObjp = rxnull_NewClientSecurityObject();
     }
-	osi_assert(secObjp != NULL);
+    osi_assert(secObjp != NULL);
     tcp->callp = rx_NewConnection(serverp->addr.sin_addr.s_addr,
                                   port,
                                   serviceID,
                                   secObjp,
                                   secIndex);
-	rx_SetConnDeadTime(tcp->callp, CM_CONN_CONNDEADTIME);
-	rx_SetConnHardDeadTime(tcp->callp, CM_CONN_HARDDEADTIME);
-	tcp->ucgen = ucellp->gen;
+    rx_SetConnDeadTime(tcp->callp, ConnDeadtimeout);
+    rx_SetConnHardDeadTime(tcp->callp, HardDeadtimeout);
+    tcp->ucgen = ucellp->gen;
     if (secObjp)
         rxs_Release(secObjp);   /* Decrement the initial refCount */
 }
 
 long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
 {
-	cm_conn_t *tcp;
+    cm_conn_t *tcp;
     cm_ucell_t *ucellp;
 
-	lock_ObtainMutex(&userp->mx);
-	lock_ObtainWrite(&cm_connLock);
-	for(tcp = serverp->connsp; tcp; tcp=tcp->nextp) {
-		if (tcp->userp == userp) break;
+    lock_ObtainMutex(&userp->mx);
+    lock_ObtainWrite(&cm_connLock);
+    for (tcp = serverp->connsp; tcp; tcp=tcp->nextp) {
+        if (tcp->userp == userp) 
+            break;
     }
-	/* find ucell structure */
+    
+    /* find ucell structure */
     ucellp = cm_GetUCell(userp, serverp->cellp);
-	if (!tcp) {
-		tcp = malloc(sizeof(*tcp));
+    if (!tcp) {
+        cm_GetServer(serverp);
+        tcp = malloc(sizeof(*tcp));
         memset(tcp, 0, sizeof(*tcp));
         tcp->nextp = serverp->connsp;
         serverp->connsp = tcp;
         cm_HoldUser(userp);
         tcp->userp = userp;
         lock_InitializeMutex(&tcp->mx, "cm_conn_t mutex");
+        lock_ObtainMutex(&tcp->mx);
         tcp->serverp = serverp;
-		tcp->cryptlevel = rxkad_clear;
-		cm_NewRXConnection(tcp, ucellp, serverp);
-		tcp->refCount = 1;
-    }
-	else {
-		if ((tcp->ucgen < ucellp->gen) || (tcp->cryptlevel != cryptall))
-		{
-			rx_DestroyConnection(tcp->callp);
-			cm_NewRXConnection(tcp, ucellp, serverp);
-		}
+        tcp->cryptlevel = rxkad_clear;
+        cm_NewRXConnection(tcp, ucellp, serverp);
+        tcp->refCount = 1;
+        lock_ReleaseMutex(&tcp->mx);
+    } else {
+        if ((tcp->ucgen < ucellp->gen) ||
+            (tcp->cryptlevel != (cryptall ? rxkad_crypt : rxkad_clear)))
+        {
+            lock_ObtainMutex(&tcp->mx);
+            rx_DestroyConnection(tcp->callp);
+            cm_NewRXConnection(tcp, ucellp, serverp);
+            lock_ReleaseMutex(&tcp->mx);
+        }
         tcp->refCount++;
-	}
-	lock_ReleaseWrite(&cm_connLock);
+    }
+    lock_ReleaseWrite(&cm_connLock);
     lock_ReleaseMutex(&userp->mx);
 
-	/* return this pointer to our caller */
+    /* return this pointer to our caller */
     osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%x", (long) tcp);
-	*connpp = tcp;
+    *connpp = tcp;
 
     return 0;
 }
@@ -523,14 +590,27 @@ long cm_Conn(struct cm_fid *fidp, struct cm_user *userp, cm_req_t *reqp,
 {
 	long code;
 
-	cm_serverRef_t *serversp;
+	cm_serverRef_t **serverspp;
 
-	code = cm_GetServerList(fidp, userp, reqp, &serversp);
+	code = cm_GetServerList(fidp, userp, reqp, &serverspp);
 	if (code) {
 		*connpp = NULL;
 		return code;
 	}
 
-	code = cm_ConnByMServers(serversp, userp, reqp, connpp);
-        return code;
+	code = cm_ConnByMServers(*serverspp, userp, reqp, connpp);
+    cm_FreeServerList(serverspp);
+    return code;
 }
+
+extern struct rx_connection * 
+cm_GetRxConn(cm_conn_t *connp)
+{
+    struct rx_connection * rxconn;
+    lock_ObtainMutex(&connp->mx);
+    rxconn = connp->callp;
+    rx_GetConnection(rxconn);
+    lock_ReleaseMutex(&connp->mx);
+    return rxconn;
+}
+
