@@ -28,7 +28,6 @@
 smb_ioctlProc_t *smb_ioctlProcsp[SMB_IOCTL_MAXPROCS];
 
 /*extern unsigned char smb_LANadapter;*/
-extern LANA_ENUM lana_list;
 
 void smb_InitIoctl(void)
 {
@@ -182,7 +181,11 @@ long smb_IoctlRead(smb_fid_t *fidp, smb_vc_t *vcp, smb_packet_t *inp,
         userp = smb_GetUser(vcp, inp);
 
 	/* Identify tree */
-	iop->tidPathp = smb_GetTIDPath(vcp, ((smb_t *)inp)->tid);
+    code = smb_LookupTIDPath(vcp, ((smb_t *)inp)->tid, &iop->tidPathp);
+    if(code) {
+        cm_ReleaseUser(userp);
+        return CM_ERROR_NOSUCHPATH;
+    }
 
 	/* turn the connection around, if required */
 	code = smb_IoctlPrepareRead(fidp, iop, userp);
@@ -206,8 +209,8 @@ long smb_IoctlRead(smb_fid_t *fidp, smb_vc_t *vcp, smb_packet_t *inp,
 
         op = smb_GetSMBData(outp, NULL);
         *op++ = 1;
-        *op++ = count & 0xff;
-        *op++ = (count >> 8) & 0xff;
+        *op++ = (char)(count & 0xff);
+        *op++ = (char)((count >> 8) & 0xff);
         
 	/* now copy the data into the response packet */
         memcpy(op, iop->outCopied + iop->outAllocp, count);
@@ -267,47 +270,55 @@ done:
 long smb_IoctlV3Read(smb_fid_t *fidp, smb_vc_t *vcp, smb_packet_t *inp, smb_packet_t *outp)
 {
 	smb_ioctl_t *iop;
-        long count;
-        long code;
-        long leftToCopy;
-        char *op;
-        cm_user_t *userp;
+    long count;
+    long code;
+    long leftToCopy;
+    char *op;
+    cm_user_t *userp;
+    smb_user_t *uidp;
 
-        iop = fidp->ioctlp;
-        count = smb_GetSMBParm(inp, 5);
+    iop = fidp->ioctlp;
+    count = smb_GetSMBParm(inp, 5);
 	
 	userp = smb_GetUser(vcp, inp);
+	osi_assert(userp != 0);
 
-	{
-		smb_user_t *uidp;
-
-		uidp = smb_FindUID(vcp, ((smb_t *)inp)->uid, 0);
-		if (uidp && uidp->unp)
-		    osi_Log3(afsd_logp, "Ioctl uid %d user %x name %s",
-			     uidp->userID, userp,
-			     osi_LogSaveString(afsd_logp, uidp->unp->name));
-		else {
-			if (uidp)
+    uidp = smb_FindUID(vcp, ((smb_t *)inp)->uid, 0);
+    iop->uidp = uidp;
+    if (uidp && uidp->unp)
+        osi_Log3(afsd_logp, "Ioctl uid %d user %x name %s",
+                  uidp->userID, userp,
+                  osi_LogSaveString(afsd_logp, uidp->unp->name));
+    else {
+        if (uidp)
 		    osi_Log2(afsd_logp, "Ioctl uid %d user %x no name",
-			     uidp->userID, userp);
-			else
+                      uidp->userID, userp);
+        else
 		    osi_Log1(afsd_logp, "Ioctl no uid user %x no name",
-			     userp);
-		}
-		smb_ReleaseUID(uidp);
-	}
+                     userp);
+    }
 
-	iop->tidPathp = smb_GetTIDPath(vcp, ((smb_t *)inp)->tid);
+	code = smb_LookupTIDPath(vcp, ((smb_t *)inp)->tid, &iop->tidPathp);
+    if(code) {
+        smb_ReleaseUID(uidp);
+        cm_ReleaseUser(userp);
+        smb_ReleaseFID(fidp);
+        return CM_ERROR_NOSUCHPATH;
+    }
 
 	code = smb_IoctlPrepareRead(fidp, iop, userp);
-        if (code) {
+    if (uidp) {
+        iop->uidp = 0;
+        smb_ReleaseUID(uidp);
+    }
+    if (code) {
 		cm_ReleaseUser(userp);
-                smb_ReleaseFID(fidp);
+        smb_ReleaseFID(fidp);
 		return code;
-        }
+    }
 
 	leftToCopy = (iop->outDatap - iop->outAllocp) - iop->outCopied;
-        if (count > leftToCopy) count = leftToCopy;
+    if (count > leftToCopy) count = leftToCopy;
         
 	/* 0 and 1 are reserved for request chaining, were setup by our caller,
          * and will be further filled in after we return.
@@ -382,13 +393,21 @@ long smb_IoctlReadRaw(smb_fid_t *fidp, smb_vc_t *vcp, smb_packet_t *inp,
 		    osi_Log3(afsd_logp, "Ioctl uid %d user %x name %s",
 			     uidp->userID, userp,
 			     osi_LogSaveString(afsd_logp, uidp->unp->name));
-		else
+		else if (uidp)
 		    osi_Log2(afsd_logp, "Ioctl uid %d user %x no name",
 			     uidp->userID, userp);
-		smb_ReleaseUID(uidp);
+        else 
+		    osi_Log1(afsd_logp, "Ioctl no uid user %x no name",
+			     userp);
+		if (uidp) smb_ReleaseUID(uidp);
 	}
 
-	iop->tidPathp = smb_GetTIDPath(vcp, ((smb_t *)inp)->tid);
+    code = smb_LookupTIDPath(vcp, ((smb_t *)inp)->tid, &iop->tidPathp);
+    if(code) {
+        cm_ReleaseUser(userp);
+        smb_ReleaseFID(fidp);
+        return CM_ERROR_NOSUCHPATH;
+    }
 
 	code = smb_IoctlPrepareRead(fidp, iop, userp);
 	if (code) {
