@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_misc.c,v 1.26 2003/07/15 23:14:24 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_misc.c,v 1.34.2.1 2004/12/07 06:12:12 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -23,17 +23,19 @@ RCSID
 #if defined(AFS_LINUX24_ENV)
 #include "h/smp_lock.h"
 #endif
-
-char *crash_addr = 0;		/* Induce an oops by writing here. */
+#if defined(AFS_LINUX26_ENV)
+#include "h/namei.h"
+#endif
 
 #if defined(AFS_LINUX24_ENV)
 /* Lookup name and return vnode for same. */
 int
-osi_lookupname_internal(char *aname, uio_seg_t seg, int followlink,
-			vnode_t ** dirvpp, struct dentry **dpp,
-			struct nameidata *nd)
+osi_lookupname(char *aname, uio_seg_t seg, int followlink,
+			vnode_t ** dirvpp, struct dentry **dpp)
 {
     int code;
+    extern struct nameidata afs_cacheNd;
+    struct nameidata *nd = &afs_cacheNd;
 
     code = ENOENT;
     if (seg == AFS_UIOUSER) {
@@ -41,8 +43,12 @@ osi_lookupname_internal(char *aname, uio_seg_t seg, int followlink,
 	    followlink ? user_path_walk(aname,
 					nd) : user_path_walk_link(aname, nd);
     } else {
+#if defined(AFS_LINUX26_ENV)
+	code = path_lookup(aname, followlink ? LOOKUP_FOLLOW : 0, nd);
+#else
 	if (path_init(aname, followlink ? LOOKUP_FOLLOW : 0, nd))
 	    code = path_walk(aname, nd);
+#endif
     }
 
     if (!code) {
@@ -56,21 +62,11 @@ osi_lookupname_internal(char *aname, uio_seg_t seg, int followlink,
     }
     return code;
 }
-#endif
-
+#else
 int
 osi_lookupname(char *aname, uio_seg_t seg, int followlink, vnode_t ** dirvpp,
 	       struct dentry **dpp)
 {
-#if defined(AFS_LINUX24_ENV)
-    struct nameidata nd;
-    int code = osi_lookupname_internal(aname, seg, followlink, dirvpp, dpp,
-				       &nd);
-    if (!code)
-	path_release(&nd);
-
-    return (code);
-#else
     struct dentry *dp = NULL;
     int code;
 
@@ -90,8 +86,8 @@ osi_lookupname(char *aname, uio_seg_t seg, int followlink, vnode_t ** dirvpp,
     }
 
     return code;
-#endif
 }
+#endif
 
 /* Intialize cache device info and fragment size for disk cache partition. */
 int
@@ -103,16 +99,12 @@ osi_InitCacheInfo(char *aname)
     extern struct osi_dev cacheDev;
     extern afs_int32 afs_fsfragsize;
     extern struct super_block *afs_cacheSBp;
-    extern struct nameidata afs_cacheNd;
-
-    code =
-	osi_lookupname_internal(aname, AFS_UIOSYS, 1, NULL, &dp,
-				&afs_cacheNd);
+    code = osi_lookupname(aname, AFS_UIOSYS, 1, NULL, &dp);
     if (code)
 	return ENOENT;
 
     cacheInode = dp->d_inode->i_ino;
-    cacheDev.dev = dp->d_inode->i_dev;
+    cacheDev.dev = dp->d_inode->i_sb->s_dev;
     afs_fsfragsize = dp->d_inode->i_sb->s_blocksize - 1;
     afs_cacheSBp = dp->d_inode->i_sb;
 
@@ -175,7 +167,6 @@ int
 osi_file_uio_rdwr(struct osi_file *osifile, uio_t * uiop, int rw)
 {
     struct file *filp = &osifile->file;
-    struct inode *ip = FILE_INODE(&osifile->file);
     KERNEL_SPACE_DECL;
     int code = 0;
     struct iovec *iov;
@@ -206,6 +197,14 @@ osi_file_uio_rdwr(struct osi_file *osifile, uio_t * uiop, int rw)
 	if (code < 0) {
 	    code = -code;
 	    break;
+	} else if (code == 0) {
+	    /*
+	     * This is bad -- we can't read any more data from the
+	     * file, but we have no good way of signaling a partial
+	     * read either.
+	     */
+	    code = EIO;
+	    break;
 	}
 
 	iov->iov_base += code;
@@ -227,10 +226,10 @@ osi_file_uio_rdwr(struct osi_file *osifile, uio_t * uiop, int rw)
  * Setup a uio struct.
  */
 void
-setup_uio(uio_t * uiop, struct iovec *iovecp, char *buf, afs_offs_t pos,
+setup_uio(uio_t * uiop, struct iovec *iovecp, const char *buf, afs_offs_t pos,
 	  int count, uio_flag_t flag, uio_seg_t seg)
 {
-    iovecp->iov_base = buf;
+    iovecp->iov_base = (char *)buf;
     iovecp->iov_len = count;
     uiop->uio_iov = iovecp;
     uiop->uio_iovcnt = 1;
@@ -248,7 +247,7 @@ setup_uio(uio_t * uiop, struct iovec *iovecp, char *buf, afs_offs_t pos,
 int
 uiomove(char *dp, int length, uio_flag_t rw, uio_t * uiop)
 {
-    int count, n;
+    int count;
     struct iovec *iov;
     int code;
 
@@ -310,21 +309,32 @@ uiomove(char *dp, int length, uio_flag_t rw, uio_t * uiop)
 void
 afs_osi_SetTime(osi_timeval_t * tvp)
 {
-    extern int (*sys_settimeofdayp) (struct timeval * tv,
-				     struct timezone * tz);
-#ifdef AFS_LINUX_64BIT_KERNEL
+#if defined(AFS_LINUX24_ENV)
+
+#if defined(AFS_LINUX26_ENV)
+    struct timespec tv;
+    tv.tv_sec = tvp->tv_sec;
+    tv.tv_nsec = tvp->tv_usec * NSEC_PER_USEC;
+#else
     struct timeval tv;
-    AFS_STATCNT(osi_SetTime);
     tv.tv_sec = tvp->tv_sec;
     tv.tv_usec = tvp->tv_usec;
-    (void)(*sys_settimeofdayp) (&tv, NULL);
+#endif
+
+    AFS_STATCNT(osi_SetTime);
+
+    do_settimeofday(&tv);
 #else
+    extern int (*sys_settimeofdayp) (struct timeval * tv,
+				     struct timezone * tz);
+
     KERNEL_SPACE_DECL;
 
     AFS_STATCNT(osi_SetTime);
 
     TO_USER_SPACE();
-    (void)(*sys_settimeofdayp) (tvp, NULL);
+    if (sys_settimeofdayp)
+	(void)(*sys_settimeofdayp) (tvp, NULL);
     TO_KERNEL_SPACE();
 #endif
 }
@@ -360,8 +370,8 @@ osi_linux_free_inode_pages(void)
 #else
 		if (ip->i_nrpages) {
 #endif
-		    printf("Failed to invalidate all pages on inode 0x%x\n",
-			   ip);
+		    printf("Failed to invalidate all pages on inode 0x%lx\n",
+			   (unsigned long)ip);
 		}
 	    }
 	}
@@ -376,24 +386,24 @@ osi_clear_inode(struct inode *ip)
 
 #if defined(AFS_LINUX24_ENV)
     if (atomic_read(&ip->i_count) > 1)
+	printf("afs_put_inode: ino %ld (0x%lx) has count %ld\n",
+	       (long)ip->i_ino, (unsigned long)ip,
+	       (long)atomic_read(&ip->i_count));
 #else
     if (ip->i_count > 1)
+	printf("afs_put_inode: ino %ld (0x%lx) has count %ld\n",
+	       (long)ip->i_ino, (unsigned long)ip, (long)ip->i_count);
 #endif
-	printf("afs_put_inode: ino %d (0x%x) has count %d\n", ip->i_ino, ip,
-	       ip->i_count);
 
     afs_InactiveVCache(vcp, credp);
     ObtainWriteLock(&vcp->lock, 504);
-#if defined(AFS_LINUX24_ENV)
-    atomic_set(&ip->i_count, 0);
-#else
-    ip->i_count = 0;
-#endif
     ip->i_nlink = 0;		/* iput checks this after calling this routine. */
+    ip->i_state = I_CLEAR;
     ReleaseWriteLock(&vcp->lock);
     crfree(credp);
 }
 
+#if !defined(AFS_LINUX26_ENV)
 /* iput an inode. Since we still have a separate inode pool, we don't want
  * to call iput on AFS inodes, since they would then end up on Linux's
  * inode_unsed list.
@@ -404,32 +414,35 @@ osi_iput(struct inode *ip)
     extern struct vfs *afs_globalVFS;
 
     AFS_GLOCK();
+
+    if (afs_globalVFS && ip->i_sb != afs_globalVFS)
+	osi_Panic("IPUT Not an afs inode\n");
+
 #if defined(AFS_LINUX24_ENV)
-    if (atomic_read(&ip->i_count) == 0
-	|| atomic_read(&ip->i_count) & 0xffff0000) {
+    if (atomic_read(&ip->i_count) == 0)
 #else
-    if (ip->i_count == 0 || ip->i_count & 0xffff0000) {
+    if (ip->i_count == 0)
 #endif
 	osi_Panic("IPUT Bad refCount %d on inode 0x%x\n",
 #if defined(AFS_LINUX24_ENV)
-		  atomic_read(&ip->i_count), ip);
+		  atomic_read(&ip->i_count),
 #else
-		  ip->i_count, ip);
+		  ip->i_count,
 #endif
-    }
-    if (afs_globalVFS && afs_globalVFS == ip->i_sb) {
+				ip);
+
 #if defined(AFS_LINUX24_ENV)
-	atomic_dec(&ip->i_count);
-	if (!atomic_read(&ip->i_count))
+    if (atomic_dec_and_test(&ip->i_count))
 #else
-	ip->i_count--;
-	if (!ip->i_count)
+    if (!--ip->i_count)
 #endif
-	    osi_clear_inode(ip);
-    } else
-	iput(ip);
+					   {
+	osi_clear_inode(ip);
+	ip->i_state = 0;
+    }
     AFS_GUNLOCK();
 }
+#endif
 
 /* check_bad_parent() : Checks if this dentry's vcache is a root vcache
  * that has its mvid (parent dir's fid) pointer set to the wrong directory
@@ -462,7 +475,7 @@ check_bad_parent(struct dentry *dp)
 	afs_lookup(pvc, dp->d_name.name, &avc, credp);
 	if (!avc || vcp != avc) {	/* bad, very bad.. */
 	    afs_Trace4(afs_iclSetp, CM_TRACE_TMP_1S3L, ICL_TYPE_STRING,
-		       "afs_linux_revalidate : bad pointer returned from afs_lookup origvc newvc dentry",
+		       "check_bad_parent: bad pointer returned from afs_lookup origvc newvc dentry",
 		       ICL_TYPE_POINTER, vcp, ICL_TYPE_POINTER, avc,
 		       ICL_TYPE_POINTER, dp);
 	}
