@@ -22,17 +22,23 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.81 2004/07/21 22:23:38 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.81.2.7 2004/12/13 19:35:35 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
 #include "afs/afs_stats.h"
 #include "afs/afs_osidnlc.h"
 #include "h/mm.h"
+#ifdef HAVE_MM_INLINE_H
+#include "h/mm_inline.h"
+#endif
 #include "h/pagemap.h"
 #if defined(AFS_LINUX24_ENV)
 #include "h/smp_lock.h"
 #endif
+#if defined(AFS_LINUX26_ENV)
+#include "h/writeback.h"
+#endif
 
 #ifdef pgoff2loff
 #define pageoff(pp) pgoff2loff((pp)->index)
@@ -81,7 +87,6 @@ afs_linux_read(struct file *fp, char *buf, size_t count, loff_t * offp)
 	if (*offp + count > afs_vmMappingEnd) {
 	    uio_t tuio;
 	    struct iovec iov;
-	    afs_size_t oldOffset = *offp;
 	    afs_int32 xfered = 0;
 
 	    if (*offp < afs_vmMappingEnd) {
@@ -142,7 +147,7 @@ static ssize_t
 afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
 {
     ssize_t code = 0;
-    int code2;
+    int code2 = 0;
     struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
     struct vrequest treq;
     cred_t *credp = crref();
@@ -249,6 +254,8 @@ afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
     return code;
 }
 
+extern int BlobScan(struct dcache * afile, afs_int32 ablob);
+
 /* This is a complete rewrite of afs_readdir, since we can make use of
  * filldir instead of afs_readdir_move. Note that changes to vcache/dcache
  * handling and use of bulkstats will need to be reflected here as well.
@@ -330,11 +337,11 @@ afs_linux_readdir(struct file *fp, void *dirbuf, filldir_t filldir)
     code = 0;
     offset = (int) fp->f_pos;
     while (1) {
-	dirpos = BlobScan(&tdc->f.inode, offset);
+	dirpos = BlobScan(tdc, offset);
 	if (!dirpos)
 	    break;
 
-	de = afs_dir_GetBlob(&tdc->f.inode, dirpos);
+	de = afs_dir_GetBlob(tdc, dirpos);
 	if (!de)
 	    break;
 
@@ -343,8 +350,8 @@ afs_linux_readdir(struct file *fp, void *dirbuf, filldir_t filldir)
 	if (de->name)
 	    len = strlen(de->name);
 	else {
-	    printf("afs_linux_readdir: afs_dir_GetBlob failed, null name (inode %x, dirpos %d)\n", 
-		   &tdc->f.inode, dirpos);
+	    printf("afs_linux_readdir: afs_dir_GetBlob failed, null name (inode %lx, dirpos %d)\n", 
+		   (unsigned long)&tdc->f.inode, dirpos);
 	    DRelease((struct buffer *) de, 0);
 	    afs_PutDCache(tdc);
 	    ReleaseReadLock(&avc->lock);
@@ -483,7 +490,6 @@ afs_linux_vma_close(struct vm_area_struct *vmap)
 	    ReleaseWriteLock(&vcp->lock);
     }
 
-  unlock_exit:
     AFS_GUNLOCK();
 }
 
@@ -646,12 +652,7 @@ afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
     int code = 0;
     struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
-#ifdef AFS_LINUX24_ENV
-    struct flock64 flock;
-#else
-    struct flock flock;
-#endif
-
+    struct AFS_FLOCK flock;
     /* Convert to a lock format afs_lockctl understands. */
     memset((char *)&flock, 0, sizeof(flock));
     flock.l_type = flp->fl_type;
@@ -839,14 +840,19 @@ afs_linux_getattr(struct vfsmount *mnt, struct dentry *dentry, struct kstat *sta
  * later on, we shouldn't have to do it until later. Perhaps in the future..
  */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,10)
+#ifdef DOP_REVALIDATE_TAKES_NAMEIDATA
+static int
+afs_linux_dentry_revalidate(struct dentry *dp, struct nameidata *nd)
+#else
 static int
 afs_linux_dentry_revalidate(struct dentry *dp, int flags)
+#endif
 #else
 static int
 afs_linux_dentry_revalidate(struct dentry *dp)
 #endif
 {
-    char *name;
+    char *name = NULL;
     cred_t *credp = crref();
     struct vrequest treq;
     struct vcache *lookupvcp = NULL;
@@ -987,13 +993,18 @@ struct dentry_operations afs_dentry_operations = {
  *
  * name is in kernel space at this point.
  */
+#ifdef IOP_CREATE_TAKES_NAMEIDATA
+int
+afs_linux_create(struct inode *dip, struct dentry *dp, int mode,
+		 struct nameidata *nd)
+#else
 int
 afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
+#endif
 {
     int code;
     cred_t *credp = crref();
     struct vattr vattr;
-    enum vcexcl excl;
     const char *name = dp->d_name.name;
     struct inode *ip;
 
@@ -1005,7 +1016,7 @@ afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
 #endif
     AFS_GLOCK();
     code =
-	afs_create(ITOAFS(dip), name, &vattr, NONEXCL, mode,
+	afs_create(ITOAFS(dip), (char *)name, &vattr, NONEXCL, mode,
 		   (struct vcache **)&ip, credp);
 
     if (!code) {
@@ -1047,8 +1058,14 @@ afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
 
 /* afs_linux_lookup */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,10)
+#ifdef IOP_LOOKUP_TAKES_NAMEIDATA
+struct dentry *
+afs_linux_lookup(struct inode *dip, struct dentry *dp,
+		 struct nameidata *nd)
+#else
 struct dentry *
 afs_linux_lookup(struct inode *dip, struct dentry *dp)
+#endif
 #else
 int
 afs_linux_lookup(struct inode *dip, struct dentry *dp)
@@ -1085,6 +1102,12 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
 	    printk
 		("afs_linux_lookup: ip->i_mode 0x%x  dp->d_name.name %s  code %d\n",
 		 ip->i_mode, dp->d_name.name, code);
+#ifdef STRUCT_INODE_HAS_I_SECURITY
+       if (ip->i_security == NULL) {
+           if (security_inode_alloc(ip))
+               panic("afs_linux_lookup: Cannot allocate inode security");
+       }
+#endif
 #else
 	if (S_ISDIR(ip->i_mode))
 	    ip->i_op = &afs_dir_iops;
@@ -1140,18 +1163,63 @@ afs_linux_link(struct dentry *olddp, struct inode *dip, struct dentry *newdp)
 int
 afs_linux_unlink(struct inode *dip, struct dentry *dp)
 {
-    int code;
+    int code = EBUSY;
     cred_t *credp = crref();
     const char *name = dp->d_name.name;
+    struct vcache *tvc = ITOAFS(dp->d_inode);
 
 #if defined(AFS_LINUX26_ENV)
     lock_kernel();
 #endif
+    if (((VREFCOUNT(tvc) > 0) && tvc->opens > 0)
+				&& !(tvc->states & CUnlinked)) {
+	struct dentry *__dp;
+	char *__name;
+	extern char *afs_newname();
+
+	__dp = NULL;
+	__name = NULL;
+	do {
+	    dput(__dp);
+
+	    AFS_GLOCK();
+	    if (__name)
+		osi_FreeSmallSpace(__name);
+	    __name = afs_newname();
+	    AFS_GUNLOCK();
+
+	    __dp = lookup_one_len(__name, dp->d_parent, strlen(__name));
+		
+	    if (IS_ERR(__dp))
+		goto out;
+	} while (__dp->d_inode != NULL);
+
+	AFS_GLOCK();
+	code = afs_rename(ITOAFS(dip), dp->d_name.name, ITOAFS(dip), __dp->d_name.name, credp);
+	if (!code) {
+            tvc->mvid = __name;
+            crhold(credp);
+            if (tvc->uncred) {
+                crfree(tvc->uncred);
+            }
+            tvc->uncred = credp;
+	    tvc->states |= CUnlinked;
+	}
+	AFS_GUNLOCK();
+
+	if (!code)
+	    d_move(dp, __dp);
+	dput(__dp);
+
+	goto out;
+    }
+
     AFS_GLOCK();
     code = afs_remove(ITOAFS(dip), name, credp);
     AFS_GUNLOCK();
     if (!code)
 	d_drop(dp);
+out:
 #if defined(AFS_LINUX26_ENV)
     unlock_kernel();
 #endif
@@ -1467,8 +1535,13 @@ afs_linux_readpage(struct file *fp, struct page *pp)
 }
 
 #if defined(AFS_LINUX24_ENV)
+#ifdef AOP_WRITEPAGE_TAKES_WRITEBACK_CONTROL
+int
+afs_linux_writepage(struct page *pp, struct writeback_control *wbc)
+#else
 int
 afs_linux_writepage(struct page *pp)
+#endif
 {
     struct address_space *mapping = pp->mapping;
     struct inode *inode;
@@ -1476,6 +1549,16 @@ afs_linux_writepage(struct page *pp)
     unsigned offset = PAGE_CACHE_SIZE;
     long status;
 
+#if defined(AFS_LINUX26_ENV)
+    if (PageReclaim(pp)) {
+	return WRITEPAGE_ACTIVATE;
+    }
+#else
+    if (PageLaunder(pp)) {
+	return(fail_writepage(pp));
+    }
+#endif
+
     inode = (struct inode *)mapping->host;
     end_index = inode->i_size >> PAGE_CACHE_SHIFT;
 
@@ -1503,8 +1586,13 @@ afs_linux_writepage(struct page *pp)
 /* afs_linux_permission
  * Check access rights - returns error if can't check or permission denied.
  */
+#ifdef IOP_PERMISSION_TAKES_NAMEIDATA
+int
+afs_linux_permission(struct inode *ip, int mode, struct nameidata *nd)
+#else
 int
 afs_linux_permission(struct inode *ip, int mode)
+#endif
 {
     int code;
     cred_t *credp = crref();
@@ -1752,7 +1840,11 @@ struct address_space_operations afs_symlink_aops = {
 struct inode_operations afs_symlink_iops = {
 #if defined(AFS_LINUX24_ENV)
   .readlink = 		page_readlink,
+#if defined(HAVE_KERNEL_PAGE_FOLLOW_LINK)
   .follow_link =	page_follow_link,
+#else
+  .follow_link =	page_follow_link_light,
+#endif
   .setattr =		afs_notify_change,
 #else
   .readlink = 		afs_linux_readlink,
