@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/dir/dir.c,v 1.16 2003/09/09 21:14:35 rees Exp $");
+    ("$Header: /cvs/openafs/src/dir/dir.c,v 1.21 2004/03/11 05:40:31 shadow Exp $");
 
 #ifdef KERNEL
 #if !defined(UKERNEL)
@@ -81,9 +81,21 @@ RCSID
 #ifndef AFS_LINUX20_ENV
 #include "netinet/in.h"
 #endif
+
+/* afs_buffer.c */
+/* These are needed because afs_prototypes.h is not included here */
+extern void *DRead();
+extern void *DNew();
+
 #else /* !defined(UKERNEL) */
 #include "afs/stds.h"
 #include "afs/sysincludes.h"
+
+/* afs_buffer.c */
+/* These are needed because afs_prototypes.h is not included here */
+extern void *DRead();
+extern void *DNew();
+
 #endif /* !defined(UKERNEL) */
 #include "afs/afs_osi.h"
 
@@ -129,9 +141,6 @@ RCSID
 #endif /* KERNEL */
 
 afs_int32 DErrno;
-char *DRelease();
-struct DirEntry *DRead();
-struct DirEntry *DNew();
 
 /* Local static prototypes */
 static struct DirEntry *FindItem(void *dir, char *ename,
