@@ -13,6 +13,7 @@
 /* rx.c */
 extern void rx_SetEpoch(afs_uint32 epoch);
 extern int rx_Init(u_int port);
+extern int rx_InitHost(u_int host, u_int port);
 #ifndef KERNEL
 extern void rxi_StartServerProcs(int nExistingProcs);
 #endif
@@ -26,6 +27,7 @@ extern void rx_SetConnDeadTime(register struct rx_connection *conn,
 			       register int seconds);
 extern void rxi_CleanupConnection(struct rx_connection *conn);
 extern void rxi_DestroyConnection(register struct rx_connection *conn);
+extern void rx_GetConnection(register struct rx_connection *conn);
 extern void rx_DestroyConnection(register struct rx_connection *conn);
 extern struct rx_call *rx_NewCall(register struct rx_connection *conn);
 extern int rxi_HasActiveCalls(register struct rx_connection *aconn);
@@ -47,12 +49,11 @@ extern void rx_WakeupServerProcs(void);
 extern struct rx_call *rx_GetCall(int tno, struct rx_service *cur_service,
 				  osi_socket * socketp);
 extern void rx_SetArrivalProc(register struct rx_call *call,
-			      register VOID(*proc) (register struct rx_call *
+			      register void (*proc) (register struct rx_call *
 						    call,
-						    register struct
-						    multi_handle * mh,
+						    register VOID * mh,
 						    register int index),
-			      register VOID * handle, register VOID * arg);
+			      register VOID * handle, register int arg);
 extern afs_int32 rx_EndCall(register struct rx_call *call, afs_int32 rc);
 extern void rx_Finalize(void);
 extern void rxi_PacketsUnWait(void);
@@ -127,9 +128,10 @@ extern struct rx_packet *rxi_SendAck(register struct rx_call *call, register str
 				     *optionalPacket, int serial, int reason,
 				     int istack);
 extern void rxi_StartUnlocked(struct rxevent *event,
-			      register struct rx_call *call, int istack);
+			      register struct rx_call *call,
+			      void *arg1, int istack);
 extern void rxi_Start(struct rxevent *event, register struct rx_call *call,
-		      int istack);
+		      void *arg1, int istack);
 extern void rxi_Send(register struct rx_call *call,
 		     register struct rx_packet *p, int istack);
 #ifdef RX_ENABLE_LOCKS
@@ -149,7 +151,7 @@ extern void rxi_SendDelayedCallAbort(struct rxevent *event,
 				     char *dummy);
 extern void rxi_ChallengeEvent(struct rxevent *event,
 			       register struct rx_connection *conn,
-			       void *atries);
+			       void *arg1, int atries);
 extern void rxi_ChallengeOn(register struct rx_connection *conn);
 extern void rxi_ComputeRoundTripTime(register struct rx_packet *p,
 				     register struct clock *sentp,
@@ -283,6 +285,8 @@ at another time. */
 #else
 extern struct rxevent *rxevent_Post(struct clock *when, void (*func) (),
 				    void *arg, void *arg1);
+extern struct rxevent *rxevent_Post2(struct clock *when, void (*func) (),
+				    void *arg, void *arg1, int arg2);
 #endif
 extern void shutdown_rxevent(void);
 extern struct rxepoch *rxepoch_Allocate(struct clock *when);
@@ -313,6 +317,7 @@ extern int rxk_initDone;
 extern int rxk_DelPort(u_short aport);
 extern void rxk_shutdownPorts(void);
 extern osi_socket rxi_GetUDPSocket(u_short port);
+extern osi_socket rxi_GetHostUDPSocket(u_int host, u_short port);
 extern void osi_Panic();
 extern int osi_utoa(char *buf, size_t len, unsigned long val);
 extern void rxi_InitPeerParams(register struct rx_peer *pp);
@@ -414,7 +419,7 @@ extern struct multi_handle *multi_Init(struct rx_connection **conns,
 				       register int nConns);
 extern int multi_Select(register struct multi_handle *mh);
 extern void multi_Ready(register struct rx_call *call,
-			register struct multi_handle *mh, register int index);
+			register VOID *mh, register int index);
 extern void multi_Finalize(register struct multi_handle *mh);
 extern void multi_Finalize_Ignore(register struct multi_handle *mh);
 
